<?php

namespace Drupal\commerce_ajax_atc\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;
use Drupal\commerce_cart\Event\CartEntityAddEvent;
use Drupal\commerce_cart\Event\CartEvents;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\OpenModalDialogCommand;
use Drupal\Core\Url;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\colorbox_load\OpenCommand;

/**
 * Event subscriber to display a popup when items are added to cart via AJAX.
 */
class KernelCartSubscriber implements EventSubscriberInterface {

  /**
   * The entity that was added to the cart.
   *
   * @var \Drupal\commerce_product\Entity\ProductVariationInterface
   */
  protected $purchasedEntity;

  /**
   * EntityTypeManager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new KernelCartSubscriber object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Used to display the rendered product_variation entity.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * Adds the popup confirmation message on page.
   *
   * @param \Symfony\Component\HttpKernel\Event\ResponseEvent $event
   *   The response event.
   */
  public function onResponse(ResponseEvent $event) {
    $config = \Drupal::config('commerce_ajax_atc.settings');
    $uses_modal = $config->get('use_modal');
    $uses_template = $config->get('use_template');
    if ($uses_template) {
      $response = $event->getResponse();

      // We only care if this happened after an entity was added to the cart.
      if (!$this->purchasedEntity) {
        return;
      }

      // We only care about AJAX responses.
      if (!$response instanceof AjaxResponse) {
        return;
      }

      // Render the status message and the entity.
      $view_builder = $this->entityTypeManager->getViewBuilder('commerce_product_variation');
      $product_variation = $view_builder->view($this->purchasedEntity, 'commerce_ajax_atc_popup');
      $title = $config->get('ajax_modal_title') ?: t('Successfully added!');
      $content = [
        '#theme' => 'commerce_ajax_atc_popup',
        '#product_variation' => $product_variation,
        '#product_variation_entity' => $this->purchasedEntity,
        '#success_message' => $config->get('success_message') ?: t('Your item has been added to your cart.'),
        '#cart_url' => Url::fromRoute('commerce_cart.page')->toString(),
        '#cart_link_text' => $config->get('cart_link_text') ?: t('View your cart.'),
        '#close_modal_url' => t('<a class=use-ajax href=":url">@close_text</a>', [
          ':url' => Url::fromRoute('commerce_ajax_atc.closeModal')->toString(),
          '@close_text' => $config->get('close_text') ?: t('Close'),
        ]),
      ];
      $options = [
        'width' => $config->get('ajax_modal_width'),
        'height' => $config->get('ajax_modal_height'),
      ];

      // TODO close links don't work in colorbox.
      $uses_colorbox_load = $config->get('pop_up_type') == 'colorbox';
      if ($uses_colorbox_load) {
        unset($content['#close_modal_url']);
        $rendered_content = \Drupal::service('renderer')->renderRoot($content);
        $response->setAttachments($content['#attached']);
        $response = new AjaxResponse();
        $response->addCommand(new OpenCommand($rendered_content));
      }
      $uses_modal = $config->get('pop_up_type') == 'modal_dialog';
      if ($uses_modal) {
        $response->addCommand(new OpenModalDialogCommand($title, $content, $options));
      }
      $event->setResponse($response);
    }

  }

  /**
   * Initializes the purchased entity.
   *
   * @param \Drupal\commerce_cart\Event\CartEntityAddEvent $event
   *   The add to cart event.
   */
  public function onAddToCart(CartEntityAddEvent $event) {
    $this->purchasedEntity = $event->getEntity();
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      KernelEvents::RESPONSE => 'onResponse',
      CartEvents::CART_ENTITY_ADD => 'onAddToCart',
    ];
  }

}
