<?php

namespace Drupal\commerce_ajax_atc\Form;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Path\PathValidator;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class AjaxSettingsForm.
 */
class AjaxSettingsForm extends ConfigFormBase {

  const AJAX_MODAL_INPUT_SIZE = 5;

  /**
   * Drupal\Core\Config\ConfigFactory definition.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * Drupal\Core\Path\PathValidator definition.
   *
   * @var \Drupal\Core\Path\PathValidator
   */
  protected $pathValidator;

  /**
   * Constructs a new AjaxSettingsForm object.
   */
  public function __construct(
    ConfigFactory $config_factory,
    PathValidator $path_validator
  ) {
    parent::__construct($config_factory);
    $this->configFactory = $config_factory;
    $this->pathValidator = $path_validator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('path.validator')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ajax_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->configFactory->get('commerce_ajax_atc.settings');
    $form['global_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Global settings'),
    ];
    $pop_up_options = [
      'non_modal' => $this->t('Non-modal message'),
      'modal_dialog' => $this->t('Modal dialog pop-up'),
    ];
    // If colorbox load is installed, create a colorbox option.
    if (\Drupal::moduleHandler()->moduleExists('colorbox_load')) {
      $pop_up_options['colorbox'] = t('Colorbox pop-up');
    }
    $form['global_settings']['pop_up_type'] = [
      '#type' => 'radios',
      '#title' => $this->t('Confirmation type'),
      '#default_value' => array_key_exists($config->get('pop_up_type'), $pop_up_options) ? $config->get('pop_up_type') : NULL,
      '#options' => $pop_up_options,
      '#description' => $this->t('Choose the type of ajax add to cart confirmation.'),
      '#required' => TRUE,
    ];
    $form['non_modal_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Non-modal settings'),
      '#states' => [
        'visible' => [
          ':input[name="pop_up_type"]' => ['value' => 'non_modal'],
        ],
      ],
    ];
    $form['non_modal_settings']['non_modal_message'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Non-modal message'),
      '#default_value' => $config->get('non_modal_message'),
      '#description' => $this->t('Enter the message for the non-modal confirmation message. If left blank the title will be "Successfully Added".'),
    ];
    $form['use_template'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable the commerce-ajax-add-to-cart-popup.html.twig template for the modal popup.'),
      '#default_value' => $config->get('use_template'),
      '#description' => $this->t('The variation can be rendered within the template using the Ajax Add to Cart Popup view mode.'),
      '#states' => [
        'visible' => [
          ':input[name="pop_up_type"]' => [
            ['value' => 'modal_dialog'],
            'or',
            ['value' => 'colorbox'],
          ],
        ],
      ],
    ];
    // Modal dialog settings.
    $form['ajax_modal_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Modal settings'),
      '#states' => [
        'visible' => [
          ':input[name="pop_up_type"]' => ['value' => 'modal_dialog'],
        ],
      ],
    ];
    $form['ajax_modal_settings']['ajax_modal_title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Modal title'),
      '#default_value' => $config->get('ajax_modal_title'),
      '#description' => $this->t('Enter the title for the modal window. If left blank the title will be "Successfully Added".'),
    ];
    $form['ajax_modal_settings']['ajax_modal_width'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Modal width'),
      '#default_value' => $config->get('ajax_modal_width'),
      '#size' => self::AJAX_MODAL_INPUT_SIZE,
      '#field_suffix' => ' px',
    ];
    $form['ajax_modal_settings']['ajax_modal_height'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Modal height'),
      '#default_value' => $config->get('ajax_modal_height'),
      '#size' => self::AJAX_MODAL_INPUT_SIZE,
      '#field_suffix' => ' px',
    ];
    // Colorbox settings.
    $form['colorbox_settings']= [
      '#type' => 'fieldset',
      '#title' => $this->t('Colorbox settings'),
      '#states' => [
        'visible' => [
          ':input[name="pop_up_type"]' => ['value' => 'colorbox'],
        ],
      ],
    ];
    $form['colorbox_settings']['colorbox_width'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Colorbox width'),
      '#default_value' => $config->get('colorbox_width'),
      '#size' => self::AJAX_MODAL_INPUT_SIZE,
      '#field_suffix' => ' px',
    ];
    $form['colorbox_settings']['colorbox_height'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Colorbox height'),
      '#default_value' => $config->get('colorbox_height'),
      '#size' => self::AJAX_MODAL_INPUT_SIZE,
      '#field_suffix' => ' px',
    ];
    // Twig template settings.
    $form['twig_template_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Twig template settings'),
      '#states' => [
        'visible' => [
          ':input[name="use_template"]' => ['checked' => TRUE],
        ],
      ],
    ];
    $form['twig_template_settings']['success_message'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Success Message'),
      '#default_value' => $config->get('success_message'),
      '#description' => $this->t('Enter the success message. If left blank the title will be "Your item has been added to your cart."'),
    ];
    $form['twig_template_settings']['cart_link_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Cart link text'),
      '#default_value' => $config->get('cart_link_text'),
      '#description' => $this->t('Enter the cart link text. If left blank the link text will be "View your cart."'),
    ];
    $form['twig_template_settings']['close_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Close modal text'),
      '#default_value' => $config->get('close_text'),
      '#description' => $this->t('Enter the text for the close modal link. If left blank the link text will be "Close"'),
    ];
    // Enable ajax for the psuedo field from commerce_variation_cart_form module.
    if (\Drupal::moduleHandler()->moduleExists('commerce_variation_cart_form')) {
      $form['variation_cart_form_settings']['enable_variation_cart_form_ajax'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Enable Commerce Variation Cart Form Ajax'),
        '#default_value' => $config->get('enable_variation_cart_form_ajax'),
        '#description' => $this->t('Enable Ajax specifically for the Commerce Variation Cart Form module.'),
      ];
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->configFactory->getEditable('commerce_ajax_atc.settings');
    $config
      ->set('pop_up_type', $form_state->getValue('pop_up_type'))
      ->set('non_modal_message', $form_state->getValue('non_modal_message'))
      ->set('use_template', $form_state->getValue('use_template'))
      ->set('ajax_modal_title', $form_state->getValue('ajax_modal_title'))
      ->set('ajax_modal_width', $form_state->getValue('ajax_modal_width') ?: '400')
      ->set('ajax_modal_height', $form_state->getValue('ajax_modal_height') ?: '300')
      ->set('colorbox_width', $form_state->getValue('colorbox_width') ?: '400')
      ->set('colorbox_height', $form_state->getValue('colorbox_height') ?: '300')
      ->set('success_message', $form_state->getValue('success_message'))
      ->set('cart_link_text', $form_state->getValue('cart_link_text'))
      ->set('close_text', $form_state->getValue('close_text'))
      ->set('enable_variation_cart_form_ajax', $form_state->getValue('enable_variation_cart_form_ajax'))
      ->save();
    parent::submitForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'commerce_ajax_atc.settings',
    ];
  }

}
